from enum import Enum


class ProvinceOrTerritory(Enum):
    ALBERTA = 'AB'
    BRITISH_COLUMBIA = 'BC'
    MANITOBA = 'MB'
    NEW_BRUNSWICK = 'NB'
    NEWFOUNDLAND = 'NL'
    NOVA_SCOTIA = 'NS'
    NORTHWEST_TERRITORIES = 'NT'
    NUNAVUT = 'NU'
    ONTARIO = 'ON'
    PRINCE_EDWARD_ISLAND = 'PE'
    QUEBEC = 'QC'
    SASKATCHEWAN = 'SK'
    YUKON = 'YK'

